@@ -21,7 +21,6 @@
 
 #pragma once
 
-#include <dazzle.h>
 #include <webkit2/webkit2.h>
 
 #include "ephy-downloads-manager.h"
@@ -35,7 +34,7 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_EMBED_SHELL (ephy_embed_shell_get_type ())
 
-G_DECLARE_DERIVABLE_TYPE (EphyEmbedShell, ephy_embed_shell, EPHY, EMBED_SHELL, DzlApplication)
+G_DECLARE_DERIVABLE_TYPE (EphyEmbedShell, ephy_embed_shell, EPHY, EMBED_SHELL, GtkApplication)
 
 typedef enum
 {
@@ -50,7 +49,7 @@ typedef enum
 
 struct _EphyEmbedShellClass
 {
-  DzlApplicationClass parent_class;
+  GtkApplicationClass parent_class;
 
   void    (* restored_window)  (EphyEmbedShell *shell);
 };
