@@ -21,8 +21,6 @@
 #include "config.h"
 #include "ephy-synchronizable.h"
 
-#include <math.h>
-
 G_DEFINE_INTERFACE (EphySynchronizable, ephy_synchronizable, JSON_TYPE_SERIALIZABLE);
 
 static void
@@ -47,7 +45,7 @@ ephy_synchronizable_get_id (EphySynchronizable *synchronizable)
 {
   EphySynchronizableInterface *iface;
 
-  g_return_val_if_fail (EPHY_IS_SYNCHRONIZABLE (synchronizable), NULL);
+  g_assert (EPHY_IS_SYNCHRONIZABLE (synchronizable));
 
   iface = EPHY_SYNCHRONIZABLE_GET_IFACE (synchronizable);
   return iface->get_id (synchronizable);
@@ -61,12 +59,12 @@ ephy_synchronizable_get_id (EphySynchronizable *synchronizable)
  *
  * Return value: @synchronizable's last modification time
  **/
-gint64
+double
 ephy_synchronizable_get_server_time_modified (EphySynchronizable *synchronizable)
 {
   EphySynchronizableInterface *iface;
 
-  g_return_val_if_fail (EPHY_IS_SYNCHRONIZABLE (synchronizable), 0);
+  g_assert (EPHY_IS_SYNCHRONIZABLE (synchronizable));
 
   iface = EPHY_SYNCHRONIZABLE_GET_IFACE (synchronizable);
   return iface->get_server_time_modified (synchronizable);
@@ -81,11 +79,11 @@ ephy_synchronizable_get_server_time_modified (EphySynchronizable *synchronizable
  **/
 void
 ephy_synchronizable_set_server_time_modified (EphySynchronizable *synchronizable,
-                                              gint64              server_time_modified)
+                                              double              server_time_modified)
 {
   EphySynchronizableInterface *iface;
 
-  g_return_if_fail (EPHY_IS_SYNCHRONIZABLE (synchronizable));
+  g_assert (EPHY_IS_SYNCHRONIZABLE (synchronizable));
 
   iface = EPHY_SYNCHRONIZABLE_GET_IFACE (synchronizable);
   iface->set_server_time_modified (synchronizable, server_time_modified);
@@ -111,8 +109,8 @@ ephy_synchronizable_to_bso (EphySynchronizable  *synchronizable,
 {
   EphySynchronizableInterface *iface;
 
-  g_return_val_if_fail (EPHY_IS_SYNCHRONIZABLE (synchronizable), NULL);
-  g_return_val_if_fail (bundle, NULL);
+  g_assert (EPHY_IS_SYNCHRONIZABLE (synchronizable));
+  g_assert (bundle);
 
   iface = EPHY_SYNCHRONIZABLE_GET_IFACE (synchronizable);
   return iface->to_bso (synchronizable, bundle);
@@ -153,9 +151,9 @@ ephy_synchronizable_from_bso (JsonNode            *bso,
   const char *payload = NULL;
   double server_time_modified;
 
-  g_return_val_if_fail (bso, NULL);
-  g_return_val_if_fail (bundle, NULL);
-  g_return_val_if_fail (is_deleted, NULL);
+  g_assert (bso);
+  g_assert (bundle);
+  g_assert (is_deleted);
 
   json = json_node_get_object (bso);
   if (!json) {
@@ -193,7 +191,7 @@ ephy_synchronizable_from_bso (JsonNode            *bso,
   }
 
   ephy_synchronizable_set_server_time_modified (EPHY_SYNCHRONIZABLE (object),
-                                                ceil (server_time_modified));
+                                                server_time_modified);
 
 out:
   if (node)
@@ -229,8 +227,8 @@ ephy_synchronizable_default_to_bso (EphySynchronizable  *synchronizable,
   char *serialized;
   char *payload;
 
-  g_return_val_if_fail (EPHY_IS_SYNCHRONIZABLE (synchronizable), NULL);
-  g_return_val_if_fail (bundle, NULL);
+  g_assert (EPHY_IS_SYNCHRONIZABLE (synchronizable));
+  g_assert (bundle);
 
   serialized = json_gobject_to_data (G_OBJECT (synchronizable), NULL);
   payload = ephy_sync_crypto_encrypt_record (serialized, bundle);
