@@ -40,6 +40,7 @@
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "ephy-find-toolbar.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-gui.h"
 #include "ephy-header-bar.h"
 #include "ephy-history-dialog.h"
@@ -562,8 +563,8 @@ window_cmd_show_about (GSimpleAction *action,
   GKeyFile *key_file;
   GBytes *bytes;
   GError *error = NULL;
-  char **list, **authors, **contributors, **past_authors, **artists, **documenters;
-  gsize n_authors, n_contributors, n_past_authors, n_artists, n_documenters, i, j;
+  char **list, **authors, **maintainers, **past_maintainers, **contributors, **artists, **documenters;
+  gsize n_authors, n_maintainers, n_past_maintainers, n_contributors, n_artists, n_documenters, i, j;
 
   key_file = g_key_file_new ();
   bytes = g_resources_lookup_data ("/org/gnome/epiphany/about.ini", 0, NULL);
@@ -574,12 +575,14 @@ window_cmd_show_about (GSimpleAction *action,
   }
   g_bytes_unref (bytes);
 
-  list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Authors",
+  list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Author",
                                      &n_authors, NULL);
+  maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Maintainers",
+                                            &n_maintainers, NULL);
+  past_maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "PastMaintainers",
+                                                 &n_past_maintainers, NULL);
   contributors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Contributors",
                                              &n_contributors, NULL);
-  past_authors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "PastAuthors",
-                                             &n_past_authors, NULL);
 
 #define APPEND(_to, _from) \
   _to[i++] = g_strdup (_from);
@@ -595,19 +598,23 @@ window_cmd_show_about (GSimpleAction *action,
   }
 
   authors = g_new (char *, (list ? n_authors : 0) +
-                   (contributors ? n_contributors : 0) +
-                   (past_authors ? n_past_authors : 0) + 7 + 1);
+                   (maintainers ? n_maintainers : 0) +
+                   (past_maintainers ? n_past_maintainers : 0) +
+                   (contributors ? n_contributors : 0) + 9 + 1);
   i = 0;
   APPEND_STRV_AND_FREE (authors, list);
   APPEND (authors, "");
+  APPEND (authors, _("Current maintainers:"));
+  APPEND_STRV_AND_FREE (authors, maintainers);
+  APPEND (authors, "");
   APPEND (authors, _("Contact us at:"));
   APPEND (authors, "<epiphany-list@gnome.org>");
   APPEND (authors, "");
+  APPEND (authors, _("Past maintainers:"));
+  APPEND_STRV_AND_FREE (authors, past_maintainers);
+  APPEND (authors, "");
   APPEND (authors, _("Contributors:"));
   APPEND_STRV_AND_FREE (authors, contributors);
-  APPEND (authors, "");
-  APPEND (authors, _("Past developers:"));
-  APPEND_STRV_AND_FREE (authors, past_authors);
   authors[i++] = NULL;
 
   list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Artists", &n_artists, NULL);
@@ -639,7 +646,11 @@ window_cmd_show_about (GSimpleAction *action,
                               webkit_get_micro_version ());
 
   gtk_show_about_dialog (window ? GTK_WINDOW (window) : NULL,
+#if !TECH_PREVIEW
                          "program-name", _("Web"),
+#else
+                         "program-name", _("Epiphany Technology Preview"),
+#endif
                          "version", VERSION,
                          "copyright", "Copyright © 2002–2004 Marco Pesenti Gritti\n"
                          "Copyright © 2003–2017 The Web Developers",
@@ -658,7 +669,11 @@ window_cmd_show_about (GSimpleAction *action,
                          "translator-credits", _("translator-credits"),
                          "logo-icon-name", "org.gnome.Epiphany",
                          "website", "https://wiki.gnome.org/Apps/Web",
+#if !TECH_PREVIEW
                          "website-label", _("Web Website"),
+#else
+                         "website-label", _("Website"),
+#endif
                          "license-type", GTK_LICENSE_GPL_3_0,
                          "wrap-license", TRUE,
                          NULL);
@@ -1383,6 +1398,9 @@ window_cmd_save_as_application (GSimpleAction *action,
   char *markup;
   char *escaped_address;
 
+  if (ephy_is_running_inside_flatpak ())
+    return;
+
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
 
@@ -1817,6 +1835,55 @@ view_source_embedded (const char *uri, EphyEmbed *embed)
   gtk_widget_grab_focus (GTK_WIDGET (new_embed));
 }
 
+static void
+tmp_source_delete_cb (GObject *source, GAsyncResult *result, gpointer data)
+{
+  GFile *file = G_FILE (source);
+  GError *error = NULL;
+
+  if (!g_file_delete_finish (G_FILE (source), result, &error)) {
+    g_warning ("Failed to delete %s: %s", g_file_get_path (file), error->message);
+    g_error_free (error);
+  }
+
+  g_object_unref (file);
+}
+
+static void
+delete_tmp_source_file (GOutputStream *ostream)
+{
+  char *uri;
+  GFile *file;
+
+  g_object_get (G_OBJECT (ostream), "ephy-save-temp-source-uri", &uri, NULL);
+  file = g_file_new_for_uri (uri);
+  g_file_delete_async (file, G_PRIORITY_LOW, NULL, tmp_source_delete_cb, NULL);
+
+  g_free (uri);
+}
+
+static void
+portal_opened_cb (GObject *source, GAsyncResult *result, gpointer data)
+{
+  char *path = data;
+  GFile *file;
+  gboolean ret;
+  GError *error = NULL;
+
+  ret = ephy_open_file_via_flatpak_portal_finish (result, &error);
+  if (!ret) {
+    /* It can be canceled by the user, even without a cancellable. */
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning ("Failed to open file via flatpak portal: %s", error->message);
+    g_error_free (error);
+  }
+
+  file = g_file_new_for_path (path);
+  g_free (path);
+
+  g_file_delete_async (file, G_PRIORITY_LOW, NULL, tmp_source_delete_cb, NULL);
+}
+
 static void
 save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointer data)
 {
@@ -1828,6 +1895,7 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
   if (error) {
     g_warning ("Unable to close file: %s", error->message);
     g_error_free (error);
+    delete_tmp_source_file (ostream);
     return;
   }
 
@@ -1835,6 +1903,14 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
 
   file = g_file_new_for_uri (uri);
 
+  if (ephy_is_running_inside_flatpak ()) {
+    const char *path;
+
+    path = g_file_get_path (file);
+    ephy_open_file_via_flatpak_portal (path, NULL, portal_opened_cb, g_strdup (path));
+    goto out;
+  }
+
   if (!ephy_file_launch_handler ("text/plain", file, gtk_get_current_event_time ())) {
     /* Fallback to view the source inside the browser */
     EphyEmbed *embed;
@@ -1845,8 +1921,9 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
                                             "ephy-save-temp-source-embed");
     view_source_embedded (uri, embed);
   }
-  g_object_unref (ostream);
 
+out:
+  g_object_unref (ostream);
   g_object_unref (file);
 }
 
@@ -1901,6 +1978,7 @@ get_main_resource_data_cb (WebKitWebResource *resource, GAsyncResult *result, GO
   if (error) {
     g_warning ("Unable to get main resource data: %s", error->message);
     g_error_free (error);
+    delete_tmp_source_file (ostream);
     return;
   }
 
@@ -1960,19 +2038,26 @@ save_temp_source (EphyEmbed *embed,
 {
   GFile *file;
   char *tmp, *base;
-  const char *static_temp_dir;
 
-  static_temp_dir = ephy_file_tmp_dir ();
-  if (static_temp_dir == NULL) {
-    return;
+  if (ephy_is_running_inside_flatpak ()) {
+    /* It has to go here because the portal has no access to our tmpfs.
+     * This means we have to delete it manually! */
+    base = g_build_filename (g_get_user_cache_dir (), "tmp", "viewsourceXXXXXX", NULL);
+  } else {
+    const char *static_temp_dir;
+
+    static_temp_dir = ephy_file_tmp_dir ();
+    if (static_temp_dir == NULL)
+      return;
+
+    base = g_build_filename (static_temp_dir, "viewsourceXXXXXX", NULL);
   }
 
-  base = g_build_filename (static_temp_dir, "viewsourceXXXXXX", NULL);
   tmp = ephy_file_tmp_filename (base, "html");
   g_free (base);
-  if (tmp == NULL) {
+
+  if (tmp == NULL)
     return;
-  }
 
   file = g_file_new_for_path (tmp);
   g_file_replace_async (file, NULL, FALSE,
